@@ -295,20 +295,21 @@ static ssize_t crypto_devrandom_read_iter(struct iov_iter *iter, bool reseed)
 		i = min_t(size_t, iov_iter_count(iter), sizeof(tmp));
 		err = crypto_rng_get_bytes(rng, tmp, i);
 		if (err) {
-			ret = err;
+			ret = ret ?: err;
 			break;
 		}
 
 		copied = copy_to_iter(tmp, i, iter);
 		ret += copied;
 
-		if (!iov_iter_count(iter) || copied != sizeof(tmp))
+		if (!iov_iter_count(iter) || copied != i)
 			break;
 
-		if (need_resched()) {
+		BUILD_BUG_ON(PAGE_SIZE % sizeof(tmp) != 0);
+		if (ret % PAGE_SIZE == 0) {
 			if (signal_pending(current))
 				break;
-			schedule();
+			cond_resched();
 		}
 	}
 
@@ -317,8 +318,7 @@ static ssize_t crypto_devrandom_read_iter(struct iov_iter *iter, bool reseed)
 	else
 		crypto_put_default_rng();
 	memzero_explicit(tmp, sizeof(tmp));
-
-	return ret;
+	return ret ? ret : -EFAULT;
 }
 
 static const struct random_extrng crypto_devrandom_rng = {
